
''' Configuration file '''

# Repo to upload the updated dependencies
PANDORA_REPO = 'https://raw.github.com/pandora-auth-ros-pkg/ci-scripts/master/pandoradep/repos.yml'

# ROSinstall templates
INSTALL_TEMPLATE_SSH = "- git: {local-name: $repo_name, uri: 'git@github.com:pandora-auth-ros-pkg/$repo_name.git', version: hydro-devel}"
INSTALL_TEMPLATE_HTTPS = "- git: {local-name: $repo_name, uri: 'https://github.com/pandora-auth-ros-pkg/$repo_name.git', version: hydro-devel}"

# Git templates
GIT_TEMPLATE_SSH = 'git@github.com:pandora-auth-ros-pkg/$repo_name.git'
GIT_TEMPLATE_HTTPS = 'https://github.com/pandora-auth-ros-pkg/$repo_name.git'

# Colorscheme
colors = {'error': 'red',
          'success': 'green',
          'info': 'magenta',
          'debug': 'blue',
          'data': 'white'
          }



import os

import click
import yaml
import catkin_pkg.packages

from pandoradep import utils


@click.group()
def cli():
    ''' A tiny cli tool to manage PANDORA's dependencies. '''
    pass


@cli.command()
@click.argument('root_of_pkgs', type=click.Path(exists=True, readable=True))
def create(root_of_pkgs):
    ''' Creates a repos.yml file, mapping each package to
        the corresponding repo. [used by Jenkins]
    '''

    package_dirs = {}

    for root, dirs, files in os.walk(root_of_pkgs):
        if '.git' in dirs:
            package_dirs[root] = []

    for repo in package_dirs:
        for root, dirs, files in os.walk(repo):
            if 'package.xml' in files:
                package_dirs[repo].append(os.path.basename(root))

    for repo in package_dirs:
        package_dirs[os.path.basename(repo)] = package_dirs.pop(repo)

    click.echo(package_dirs)

    with open('repos.yml', 'w') as f:
        f.write(yaml.dump(package_dirs))


@cli.command()
@click.argument('root', type=click.Path(exists=True, readable=True))
@click.argument('repo_name', type=click.STRING)
@click.argument('repos_file', type=click.STRING)
@click.option('--env', type=click.STRING, default='JENKINS_SCRIPTS',
              help='''Specify environmental variable for
                      the scripts. The default is JENKINS_SCRIPTS
                   ''')
def update(root, repo_name, repos_file, env):
    '''Updates dependencies [used by Jenkins]'''

    with open(repos_file, 'r') as f:
        repos = yaml.safe_load(f)

    catkin_output = catkin_pkg.packages.find_packages(root)

    local_pkgs = [pkg.name for pkg in catkin_output.values()]

    if set(repos[repo_name]) == set(local_pkgs):
        click.echo(click.style('Nothing changed', fg='green'))
    else:
        click.echo(click.style('Updating packages...', fg='yellow'))
        repos[repo_name] = local_pkgs
        utils.update_upstream(repos_file, repos, env)


@cli.command()
@click.argument('directory')
@click.option('--build', is_flag=True,
              help='Return only the build dependencies')
@click.option('--run', is_flag=True, help='Return only the run dependencies')
@click.option('--git', is_flag=True, help='Return git repositories.')
@click.option('--save', type=click.Path(exists=True,
              resolve_path=True, writable=True),
              help='Download the dependencies in a specified directory.')
@click.option('--http', is_flag=True,
              help='Return git repos or rosinstall entries with https.')
@click.option('--exclude', '-x', multiple=True, default=None,
              type=click.Path(exists=True, readable=True),
              help='Exclude a directory from the scan.')
def scan(directory, http, exclude, git, save, build, run):
    ''' Scans the directory tree for dependencies. By default returns
        rosinstall entries that you can feed into the wstool.
    '''

    (build_depends, run_depends) = utils.get_dependencies(directory, exclude)
    depends = set(build_depends).union(run_depends)

    repos = utils.fetch_upstream()

    if build:
        utils.print_repos(build_depends, repos, http, git, save)
    elif run:
        utils.print_repos(run_depends, repos, http, git, save)
    else:
        utils.print_repos(depends, repos, http, git, save)



import sys
import os
import subprocess
from subprocess import check_call, CalledProcessError
from string import Template

import yaml
import requests
import click
import catkin_pkg.packages

from config import PANDORA_REPO, INSTALL_TEMPLATE_SSH, INSTALL_TEMPLATE_HTTPS, \
    GIT_TEMPLATE_SSH, GIT_TEMPLATE_HTTPS, colors


def get_dependencies(directory, excluded=None):
    ''' Fetches all the run and build dependencies '''

    build_depends = set([])
    run_depends = set([])

    pkgs = catkin_pkg.packages.find_packages(directory, excluded)

    for pkg in pkgs:
        build_depends = build_depends.union(map(str, pkgs[pkg].build_depends))
        run_depends = run_depends.union(map(str, pkgs[pkg].exec_depends))

    return (build_depends, run_depends)


def fetch_upstream():
    ''' Returns the current pandora dependencies '''

    r = requests.get(PANDORA_REPO)

    return yaml.safe_load(r.text)


def print_repos(depends, repos, http, git, save_path):
    ''' Prints dependencies using git or rosinstall templates '''

    repos_to_fetch = set([])

    if git and http:
        template = Template(GIT_TEMPLATE_HTTPS)
    elif git:
        template = Template(GIT_TEMPLATE_SSH)
    elif http:
        template = Template(INSTALL_TEMPLATE_HTTPS)
    else:
        template = Template(INSTALL_TEMPLATE_SSH)

    for dep in depends:
        for repo, packages, in repos.items():
            for package in packages:
                if dep == package:
                    repos_to_fetch.add(repo)

    if save_path:

        if http:
            template = Template(GIT_TEMPLATE_HTTPS)
        else:
            template = Template(GIT_TEMPLATE_SSH)

        click.echo(click.style('Saving dependencies in: ' + save_path,
                               fg=colors['debug']))
        click.echo()
        try:
            os.chdir(save_path)
        except OSError:
            click.echo(click.style('Invalid save path ' + save_path,
                       fg=colors['error']))

        for repo in repos_to_fetch:
            git_repo = template.substitute(repo_name=repo)
            click.echo(click.style('### Cloning ' + git_repo,
                       fg=colors['info']))
            try:
                check_call(['git', 'clone', git_repo])
            except subprocess.CalledProcessError:
                pass
    else:

        for repo in repos_to_fetch:
            click.echo(click.style(template.substitute(repo_name=repo),
                                   fg=colors['success']))


def update_upstream(output_file, content, env_var):
    ''' Updates upstream yaml file '''

    with open(output_file, 'w') as f:
        f.write(yaml.dump(content))

    scripts_path = os.getenv(env_var)

    if not scripts_path:
        raise ValueError('$' + env_var + ' is not set properly.')
    try:
        os.chdir(scripts_path)
    except OSError:
        click.echo(click.style('Error: Make sure your enviroment is set properly',
                   fg=colors['error']))

    git_commands = ["git add -u",
                    "git commit -m 'Update repos.yml'",
                    "git push origin master"
                    ]

    for cmd in git_commands:
        click.echo(click.style('+ ' + cmd, fg=colors['debug']))
        try:
            check_call(cmd, shell=True)
        except subprocess.CalledProcessError:
            click.echo(click.style('Error: ' + cmd, fg=colors['error']))



from setuptools import setup, find_packages


setup(name='pandoradep',
      packages=['pandoradep'],
      version='0.1',
      py_modules=['index'],
      description="A tiny cli tool to manage PANDORA's dependencies.",
      author='PANDORA Robotics Team',
      author_email='siderisk@auth.gr',
      url='https://github.com/pandora-auth-ros-pkg/pandoradep',
      license='BSD',
      install_requires=[
          'click',
          'catkin-pkg',
          'requests',
          'colorama',
          'pyYAML'
          ],
      entry_points='''
        [console_scripts]
        pandoradep=index:cli
      ''',
      )
